#ifndef DRIVER_H
#define DRIVER_H

#include <stdint.h>
#include "common.h"

/**
 * @defgroup Drivers Интерфейс драйверов устройств
 *
 * Драйвер устройства состоит из структуры GATE_DRIVER, функций чтения и/или
 * записи в регистр, и (необязательно) функции инициализации драйвера.
 *
 * Регистры, используемые драйвером, должны содержаться в массиве, на который
 * указывает поле registers. Количество используемых регистров задается полем
 * num_registers.
 *
 * Работа с устройством ввода/вывода заключается в записи массива данных в
 * регистры или чтения из регистра. То, как использовать массив данных,
 * переданных для записи, или как заполнять буфер данных при чтении, целиком
 * определяется драйвером устройства.
 *
 */

/**@{*/

/**
 * Прототип функции чтения данных из драйвера. Драйвер, реализующий чтение из
 * регистров, должен предоставлять функцию типа GATE_READ. Прочитанные данные
 * помещаются в буфер data. Количество байт для чтения задается переменной,
 * указатель на которую содержится в data_len. В эту же переменную помещается
 * количество фактически прочитанных байтов.
 *
 * @param reg Номер регистра.
 * @param data Указатель на буфер данных
 * @param data_len Указатель на переменную, значение которй задает количество
 *                 байт для чтения.
 *
 * @return GR_OK - если данные успешно прочитаны. Иначе - код ошибки. Функция
 * должна возвращать GR_NO_ACCESS, если чтение из запрошенного регистра не
 * поддерживается.
 */
typedef GATE_RESULT (*GATE_READ)(uint8_t reg, uint8_t* data, uint8_t* data_len);

/**
 * Прототип функции записи данных в драйвер.
 * Драйвер, реализующий запись в регистры, должен предоставлять функцию типа
 * GATE_READ.
 *
 * @param reg Номер регистра.
 * @param data Указатель на массив с данными.
 * @param data_len Количество байт для записи.
 *
 * @return GR_OK - если данные успешно записаны. Иначе - код ошибки. Функция
 *         должна возвращать GR_NO_ACCESS, если запись в запрошенный регистр
 *         не поддерживается.
 */
typedef GATE_RESULT (*GATE_WRITE)(uint8_t reg, uint8_t* data, uint8_t data_len);

/**
 * Прототип функции инициализации драйвера.
 * Функция инициализации драйвера вызывается при регистрации драйвера.
 *
 * @return Функция должна возвращать GR_OK при успешной инициализации, иначе -
 *         код ошибки
 */
typedef GATE_RESULT (*GATE_INIT)(void);

/**
 * Конфигурация драйвера устройств.
 */
typedef struct GATE_DRIVER_ {
	uint8_t* registers; /**< Массив с номерами регистров, обслуживаемых драйвером */
	uint8_t  num_registers; /**< Количество регистров */
	struct GATE_DRIVER_* next; /*<< Указатель на следующий драйвер в списке */
	GATE_READ read; /**< Функция чтения */
	GATE_WRITE write; /**< Функция записи */
	GATE_INIT init; /**< Функция инициализации драйвера */
} GATE_DRIVER;

/**
 * Регистрирует драйвер устройства.
 * @param driver Указатель на структуру с описанием драйвера
 * @return GR_OK, если драйвер был успешно добавлен. GR_DUPLICATE_REGISTER,
 *         если один из регистров уже используется другим драйвером.Если 
 *         драйвер предоставляетт функцию инициализации, возможны и другие
 *         значения. Любое значение, отличное от GR_OK, означает, что драйвер
 *         не был добавлен, и использовать его регистры невозможно.
 */
GATE_RESULT gate_driver_register(GATE_DRIVER* driver);

/**
 * Чтение из регистра.
 * @param reg Номер регистра
 * @param data Указатель на буфер данных
 * @param data_len Указатель на переменную, значение которй задает количество байт для чтения.
 * @return GR_INVALID_REGISTER - если для указанного регистра нет
 *         зарегистрированного драйвера. GR_NO_ACCESS - если драйвер, связанный
 *         с регистром, не предоставляет функцию чтения или чтение из
 *         запрошенного регистра не поддерживается. Иначе - результат выполнения
 *         функции чтения драйвера.
 *
 * @see GATE_READ
 */
GATE_RESULT gate_register_read(uint8_t reg, uint8_t* data, uint8_t* data_len);

/**
 * Запись в регистр.
 * @param reg Номер регистра.
 * @param data Указатель на массив с данными.
 * @param data_len Количество байт для записи.
 *
 * @return GR_INVALID_REGISTER - если для указанного регистра нет
 *         зарегистрированного драйвера. GR_NO_ACCESS - если драйвер, связанный
 *         с регистром, не предоставляет функцию записи или запись в запрошенный
 *         регистр не поддерживается. Иначе - результат выполнения функции
 *         записи драйвера.
 *
 * @see GATE_WRITE
 */
GATE_RESULT gate_register_write(uint8_t reg, uint8_t* data, uint8_t data_len);

/**@}*/

#endif
